/*
 * XTuxcall -- Call waiting detector for Linux (Gtk version)
 * Copyright (C) 2001-2002 Angel Luis Jimenez Martinez
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>

#include "common.h"
#include "support.h"

extern int FramesUpdate, Mode;
extern float Freq, LimitMult;
extern char Device[], HangCmd[];
extern GtkWidget * mainWin;
extern GtkWidget * confWin;
char msg[100];

/*
 * Writes message on application's status bar
 */
void statusMsg(gchar * message)
{
    GtkStatusbar * status;
    int cid;

    status= (GtkStatusbar *)lookup_widget(GTK_WIDGET(mainWin), "statusBar");
    cid= gtk_statusbar_get_context_id(status, "Main Window");

    gdk_threads_enter();
    gtk_statusbar_pop( status, cid );
    gtk_statusbar_push( status, cid, message );
    gdk_threads_leave();
}

/*
 * Shows message using a dialog
 */
void quickMsg(gchar *message) 
{

    GtkWidget *dialog, *label, *okay_button;
   
   
    dialog = gtk_dialog_new();
    label = gtk_label_new (message);
    okay_button = gtk_button_new_with_label("OK");

    gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), 5);   
   
    gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
                              GTK_SIGNAL_FUNC (gtk_widget_destroy),
                              (GtkObject *)dialog);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
                      okay_button);
    /*gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                      okay_button, FALSE, TRUE, 10 );*/


    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
                      label);
    gtk_widget_show_all (dialog);
}

/*
 * Set values for config window's widgets from global vars
 */
void showConf()
{
    GtkWidget * w;
    char str[200];

    w= lookup_widget (GTK_WIDGET(confWin), "confFrames");
    snprintf( str, 200-1, "%d", FramesUpdate );
    gtk_entry_set_text( GTK_ENTRY(w), str );

    w= lookup_widget (GTK_WIDGET(confWin), "confFrequency");
    snprintf( str, 200-1, "%.0f", Freq );
    gtk_entry_set_text( GTK_ENTRY(w), str );

    w= lookup_widget (GTK_WIDGET(confWin), "confDevice");
    gtk_entry_set_text( GTK_ENTRY(w), Device );

    w= lookup_widget (GTK_WIDGET(confWin), "confCommand");
    gtk_entry_set_text( GTK_ENTRY(w), HangCmd );

    w= lookup_widget (GTK_WIDGET(confWin), "confMode");
    if ( Mode == EU )
        gtk_option_menu_set_history( GTK_OPTION_MENU(w), 0 );
    else
        gtk_option_menu_set_history( GTK_OPTION_MENU(w), 1 );

    w= lookup_widget (GTK_WIDGET(confWin), "confMult");
    snprintf( str, 200-1, "%.2f", LimitMult );
    gtk_entry_set_text( GTK_ENTRY(w), str );
}

/*
 * Changes global var's values
 */
void applyConf( char * device, char * hangCmd, float frequency, int frames
    , int mode, float limitMult )
{
    strncpy( Device, device, FNAMELEN );
    strncpy( HangCmd, hangCmd, CMDLEN );
    Freq= frequency;
    FramesUpdate= frames;
    Mode= mode;
    LimitMult= limitMult;
}

