/*
 * Tuxcall -- Call waiting detector for Linux
 * Copyright (C) 2001-2002 Angel Luis Jimenez Martinez
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#   include <config.h>
#endif

#include <intl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"
#include "conf.h"
#include "xconf.h"
#include "xaux.h"

#define OPTLEN 50
#define VALUELEN 210
#define LINELEN (OPTLEN+VALUELEN)

#define COMMENT '#'

/*
 * Reads XTuxCall's config from fich
 */
void xLoadConfig( FILE * fich, char * device, char * hangCmd, float * freq, int * frames, enum tAction * action, int * mode, float * limitMult )
{
    char opt[OPTLEN]
       , value[VALUELEN]
       , line[LINELEN];
    int nLine= 1;   
    char * chk;

    while( fgets( line, LINELEN-1, fich ) != NULL )
    {
        if ( isValid(line) )
        {
            parseLine( line, nLine, opt, value );
/*#ifdef DEBUG
            printf( "%d:%s=%s\n", nLine, opt, value );
#endif*/

            if ( strcmp(opt, "frecuencia") == 0 )
            {
                *freq= strtod( value, &chk );
                if ( *freq==0 && value==chk )
                {
                    fprintf( stderr, _("Error: reading config file, line %d\n")
                        , nLine ); 
                    fprintf( stderr, _("Error: \"frecuencia\" must be a real number\n") );
                    exit( -1 );
                }

            }
            else if ( strcmp(opt, "frames") == 0 )
            {
                *frames= strtol( value, &chk, 10 );
                if ( (*frames==0 && value==chk) || (*frames <= 0) )
                {
                    fprintf( stderr, _("Error: reading config file, line %d\n")
                        , nLine ); 
                    fprintf( stderr, _("Error: \"frames\" must be a positive integer number\n") );
                    exit( -1 );
                }

            }
            else if ( strcmp(opt, "comando") == 0 )
            {
                strncpy( hangCmd, value, CMDLEN );
            }
            else if ( strcmp(opt, "dispositivo") == 0 )
            {
                strncpy( device, value, FNAMELEN );
            }
            else if ( strcmp(opt, "accion") == 0 )
            {
                if ( strcmp(value, "aceptar") == 0 )
                    *action= ACCEPT;
                else if ( strcmp(value, "rechazar") == 0 )
                    *action= REJECT;
                else if ( strcmp(value, "preguntar") == 0 )
                    *action= ASK;
            }
            else if ( strcmp(opt, "modo") == 0 )
            {
                if ( strcmp(value, "EU") == 0 )
                    *mode= EU;
                else if ( strcmp(value, "USA") == 0 )
                    *mode= USA;
            }        
            else if ( strcmp(opt, "limitmult") == 0 )
            {
                *limitMult= strtod( value, &chk );
                if ( *limitMult==0 && value==chk )
                {
                    fprintf( stderr, _("Error: reading config file, line %d\n")
                        , nLine ); 
                    fprintf( stderr, _("Error: \"limitmult\" must be a real number\n") );
                    exit( -1 );
                }

            }
            else
            {
                fprintf( stderr, _("Error: reading config file, line %d\n")
                        , nLine ); 
                fprintf( stderr, _("Error: Unknown option \"%s\"\n"), opt );
                exit( -1 );
            }
        }

        nLine++;
    }
}

/*
 * Saves XTuxcall config on file. Receives app's values
 */
void xSaveConf( FILE * f, char * device, char * hangCmd, float freq, int frames, enum tAction action, int mode, float limitMult )
{
    fprintf( f, "dispositivo= %s\n", device );
    fprintf( f, "comando= %s\n", hangCmd );
    fprintf( f, "frecuencia= %f\n", freq );
    fprintf( f, "frames= %d\n", frames );
    fprintf( f, "accion= " );
    switch (action)
    {
        case ACCEPT:
            fprintf( f, "aceptar\n" );
        break;    
        case REJECT:
            fprintf( f, "rechazar\n" );
        break;    
        case ASK:
            fprintf( f, "preguntar\n" );
    }        
    fprintf( f, "modo= " );
    switch (mode)
    {
        case EU:
            fprintf( f, "EU\n" );
        break;    
        case USA:
            fprintf( f, "USA\n" );
    }    
    fprintf( f, "limitmult= %f\n", limitMult );
}
