/*
 * XTuxcall -- Call waiting detector for Linux (Gtk version)
 * Copyright (C) 2001-2002 Angel Luis Jimenez Martinez
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <linux/soundcard.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <pthread.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "common.h"
#include "xcommon.h"
#include "callbacks.h"
#include "detect.h"
#include "conf.h"
#include "xconf.h"
#include "aux.h"
#include "xaux.h"
#include "dsp.h"
#include "iir.h"
#include "interface.h"
#include "support.h"

int Log= 0
  , ChunkSize= 640
  , End= 0
  , CheckPPP= 1
  , FramesUpdate= 10
  , Mode= EU;
float Silence= 0.2
  , Freq= 420
  , LimitMult= 3; /* Multiplier which defines what is a callwaiting tone */
char HangCmd[CMDLEN]= "killall -9 pppd" /* Command for hanging connection */
  ,Device[CMDLEN]= "/dev/dsp"; /* Sound input device */
enum tAction Action= ACCEPT;

void (*FuncOnCall)()= hang; /* Function executed when a call is detected */

GtkWidget * mainWin;
GtkWidget * callWin;

typedef struct
        {
            char device[FNAMELEN];
            signed short * buff;
            float * filtered;
            float * a;
            float * b;
        } DetectData;

void use()
{
    printf( _("XTuxCall - version %s\n\n"), VERSION );
    printf( _("use: xtuxcall [-n]\n") );
    printf( _("   -n              Don't check for ppp connection.\n") );
}

/*
 * Parse program arguments
 */
void parseArgs( int argc, char ** argv )
{
    int i;

    for ( i=1; i<argc; i++ )
    {
        if ( strcmp(argv[i], "-n") == 0 )
        {
            CheckPPP= 0;
        }
        else    
        {
            use();
            exit(-1);
        }
    }    
}

/*
 * Function executed after a detection step
 */
void afterStep( float chunkMean, float limit )        
{
    gdk_threads_enter();
    updateGraph( chunkMean, limit );
    gdk_threads_leave();
}

/*
 * Detect 'em all
 */
void * detect( void * ptr ) 
{
    DetectData * p= ptr;
    int dsp;
    GtkStatusbar * w;
    char msg[200];
    int cid, go;
    long readLen;

    w= (GtkStatusbar *)lookup_widget (GTK_WIDGET(mainWin), "statusBar");
    cid= gtk_statusbar_get_context_id(w, "Main Window");

    statusMsg( _("Waiting for ppp connection... ") );
#ifdef DEBUG            
    fprintf( stderr, _("Waiting for ppp connection... ") );
    fflush( stderr );
#endif

    while ( !End )
    {
        if ( connected() || !CheckPPP )
        {
            
            snprintf( msg, 200-1, _("Opening %s... "), p->device);
            statusMsg( msg );
#ifdef DEBUG            
            fprintf( stderr, _("detected.\n") );
            fprintf( stderr, msg );
            fflush( stderr );
#endif            
            dsp= initDSP( p->device, FORMAT, CHANNELS, SPEED );
            if ( dsp == -1 )
            {
                snprintf( msg , 200-1, _("Error opening %s\n"), p->device );
                fprintf( stderr, msg );
                quickMsg( msg );
                break;
            }
            statusMsg( _("Detecting...") );
#ifdef DEBUG            
            printf( _("done.\n") );
            fprintf( stderr, _("Detecting...") );
            fprintf( stderr, "\n");
#endif            


            go= 1;
            if ( Mode == EU )
                while ( go )
                {
                    readLen= readChunk( dsp, p->buff, BUFFLEN );
                    go= detectStep( p->buff, p->filtered, readLen, p->a
                        , p->b, stateDetectEU, afterStep, FuncOnCall );
                }
            else    
                while ( go )
                {
                    readLen= readChunk( dsp, p->buff, BUFFLEN );
                    go= detectStep( p->buff, p->filtered, readLen, p->a
                        , p->b, stateDetectUSA, afterStep, FuncOnCall );
                }

            close( dsp );

            /*gtk_statusbar_push( w, 1, _("Waiting for ppp conection...") );*/
            
            statusMsg( _("Waiting for ppp connection... ") );
#ifdef DEBUG            
            fprintf( stderr, _("Waiting for ppp connection... ") );
            fflush( stderr );
#endif            
        }
        else
            sleep( 5 );
    }

    return NULL;
}


int
main (int argc, char *argv[])
{
    float a[3], b[3];
    signed short * buff;
    float * filtered;
    float center, bw;
    DetectData data;
    pthread_t thread;
    char str[300];
    char confFile[FNAMELEN];
    FILE * f;

    bw= 0.00125;
    center= Freq/SPEED;
            

#ifdef ENABLE_NLS
    setlocale(LC_MESSAGES, "");
    setlocale(LC_CTYPE, "");
    bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
    textdomain (PACKAGE);
#endif

    /*gtk_set_locale();*/
    g_thread_init( NULL );
    gtk_init (&argc, &argv);

    /* Loading config */
    snprintf( confFile, FNAMELEN-1, "%s/.%s", getenv("HOME"), CONF );
    f= fopen( confFile, "r" );
    if ( f != NULL )
    {
        xLoadConfig( f, Device, HangCmd, &Freq, &FramesUpdate, &Action, &Mode
            , &LimitMult  );
        fclose( f );
    }

    parseArgs( argc, argv );


    allocateBuff( &buff, &filtered, BUFFLEN );
    iirPassband( a, b, center, bw );
            
#ifdef DEBUG    
    printf( _("device= %s\n"), Device );
    printf( _("command= %s\n"), HangCmd );
    printf( _("frequency= %f\n"), Freq );
    printf( _("mode= %s\n"), (Mode==EU)?"EU":"USA" );
    printf( "center= %f, bw= %f\n", center, bw );
    printf( "a0= %40.43f\na1= %40.43f\na2= %40.43f\nb1= %40.43f\nb2= %40.43f\n"
        , a[0], a[1], a[2], b[1], b[2] );
#endif


    add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
    add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");

    mainWin= create_mainWin();
    callWin= create_callWin();

    snprintf( str, 299, "XTuxCall - %s", VERSION );
    gtk_window_set_title (GTK_WINDOW (mainWin), str);
    gtk_widget_show (mainWin);

    strncpy(data.device, Device, FNAMELEN);
    data.buff= buff;
    data.filtered= filtered;
    data.a= a;
    data.b= b;

    pthread_create( &thread, NULL, detect, &data );

    gtk_main();

    freeBuff( buff, filtered );

    return 0;
}
